package com.agilefaqs.rft.suggestions;

import static java.util.Arrays.asList;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.agilefaqs.rft.domain.DomainNameService;
import com.agilefaqs.rft.domain.RandomNumberGenerator;
import com.agilefaqs.rft.domain.UserService;

public class EmailSuggester {
    private static final int MAX_RETRIES_FOR_RANDOM_NUMBER = 5;
    private final UserService userService;
    private final DomainNameService dns;
    private final RandomNumberGenerator randomNumber;

    public EmailSuggester(final UserService userService, final DomainNameService dns, final RandomNumberGenerator randomNumber) {
        this.userService = userService;
        this.dns = dns;
        this.randomNumber = randomNumber;
    }

    public List<String> optionsFor(final User user) {
        List<String> ids = new ArrayList<String>();
        List<String> variations = asList(user.lastName, user.countryName, user.countryMoniker, user.city);
        for (String variation : variations) {
            UserData data = new UserData(user.firstName, variation, user.lastName);
            data.addGeneratedIdTo(ids);
        }
        return ids;
    }

    private final class UserData {
        private final String firstName;
        private final String seed;
        private final String lastName;

        private UserData(final String firstName, final String seed, final String lastName) {
            this.firstName = firstName;
            this.seed = seed;
            this.lastName = lastName;
        }

        private void addGeneratedIdTo(final List<String> ids) {
            for (EmailData potential : buildAllPotentialEmailCombinations()) {
                String email = Email.create(potential.userName, potential.domain, dns);
                if (userService.isEmailAvailable(email)) {
                    ids.add(email);
                    break;
                }
            }
        }

        private List<EmailData> buildAllPotentialEmailCombinations() {
            return new ArrayList<EmailData>() {
                {
                    add(new EmailData(firstName, seed));

                    if (seed != lastName) {
                        add(new EmailData((firstName + lastName), seed));
                        add(new EmailData((firstName + lastName.charAt(0)), seed));
                    }

                    add(new EmailData((firstName + currentYear()), seed));

                    if (seed != lastName)
                        add(new EmailData((firstName + lastName.charAt(0) + currentYear()), seed));

                    for (int i = 0; i < MAX_RETRIES_FOR_RANDOM_NUMBER; ++i)
                        add(new EmailData((firstName + randomNumber.next()), seed));
                }
            };
        }

    }

    private static final class EmailData {
        private final String userName;
        private final String domain;

        private EmailData(final String userName, final String domain) {
            this.userName = userName;
            this.domain = domain;
        }

    }

    private static int currentYear() {
        return Calendar.getInstance().get(Calendar.YEAR);
    }
}
