package com.agilefaqs.rft.suggestions;

import static java.lang.String.valueOf;

public class User {
    private static final int MIN_WORD_LENGTH = 3;
    private static final int MAX_WORD_LENGTH = 15;
    private static final char[] ALLOWED_CHARS = new char[] { '-' };
    public final String firstName;
    public final String lastName;
    public final String city;
    public final String countryName;
    public final String countryMoniker;

    public User(final String firstName, final String lastName, final String city, final String countryName, final String countryMoniker) {
        this.firstName = buildValidToken(firstName);
        this.lastName = buildValidToken(lastName);
        this.city = city;
        this.countryName = countryName;
        this.countryMoniker = countryMoniker;
    }

    private static String buildValidToken(final String token) {
        String word = stripSpecialCharacters(token, ALLOWED_CHARS);
        word = appendLettersIfRequired(word);
        word = truncateLettersIfRequired(word);
        return word;
    }

    private static String truncateLettersIfRequired(String word) {
        if (word.length() > MAX_WORD_LENGTH)
            word = word.substring(0, MAX_WORD_LENGTH);
        return word;
    }

    private static String appendLettersIfRequired(String word) {
        while (word.length() < MIN_WORD_LENGTH)
            word = word + word;
        return word;
    }

    static String stripSpecialCharacters(final String sample, final char[] allowedSpecialChars) {
        String spChar = valueOf(allowedSpecialChars);
        return sample.replaceAll("(^[" + spChar + "]+|[^a-zA-Z0-9" + spChar + "]|[" + spChar + "]+$)+", "");
    }
}
