package com.agilefaqs.rft.suggestions;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Mockito.stub;

import java.util.Arrays;
import java.util.List;

import com.agilefaqs.rft.domain.DomainNameService;
import com.agilefaqs.rft.domain.UserService;

class Context {
    private String[] tokens;
    private String identity;
    private final DomainNameService dns;
    private UserService userService;
    private List<String> generatedIds;
    private String[] ids;

    public Context(final DomainNameService dns) {
        this.dns = dns;
    }

    public Context(final UserService userService, final DomainNameService dns) {
        this.userService = userService;
        this.dns = dns;
        stub(userService.isEmailAvailable((String) anyObject())).toReturn(true);
    }

    public void isACelebrityName() {
        isCelebrityName(true);
    }

    public void isARestrictedDomainName() {
        isRestrictedDomainName(tokens[0], true);
    }

    void isMadeUpOf(final String token1, final String token2) {
        assertEquals((token1 + "@" + token2 + ".com"), identity);
    }

    Context assertThat(final String identity) {
        this.identity = identity;
        return this;
    }

    void isNotARestrictedDomainName() {
        isRestrictedDomainName(tokens[0], false);
    }

    void areRestrictedDomainNames() {
        for (String token : tokens)
            isRestrictedDomainName(token, true);
    }

    private void isRestrictedDomainName(final String token, final boolean value) {
        stub(dns.isRestrictedDomainName(token)).toReturn(value);
    }

    void isNotARestrictedUserName() {
        isRestrictedUserName(tokens[0], false);
    }

    void isARestrictedUserName() {
        isRestrictedUserName(tokens[0], true);
    }

    void areRestrictedUserNames() {
        for (String token : tokens)
            isRestrictedUserName(token, true);
    }

    private void isRestrictedUserName(final String token, final boolean value) {
        stub(dns.isRestrictedUserName(token)).toReturn(value);
    }

    void isNotACelebrityName() {
        isCelebrityName(false);
    }

    private void isCelebrityName(final boolean value) {
        stub(dns.isCelebrityName(tokens[0], tokens[1])).toReturn(value);
    }

    Context assume(final String... tokens) {
        this.tokens = tokens;
        return this;
    }

    public void isAvailable() {
        isAvailable(true);
    }

    public Context assume() {
        return this;
    }

    public Context identity(final String... ids) {
        this.ids = ids;
        return this;
    }

    public void isNotAvailable() {
        isAvailable(false);
    }

    private void isAvailable(final boolean value) {
        for (String id : ids)
            stub(userService.isEmailAvailable(id)).toReturn(value);
    }

    public Context assertThat(final List<String> generatedIds) {
        this.generatedIds = generatedIds;
        return this;
    }

    public void are(final String... actualIds) {
        assertEquals(Arrays.asList(actualIds), generatedIds);
    }

    static String[][] list(final String[]... elements) {
        return elements;
    }

    static String[] _(final String... element) {
        return element;
    }
}
