package com.agilefaqs.rft.suggestions;

import static com.agilefaqs.rft.suggestions.User.stripSpecialCharacters;
import static org.junit.Assert.assertEquals;

import org.junit.Test;

public class UserInfoTest {
    private final char[] allowedChars = new char[] { '-', '@' };

    @Test
    public void shouldTrimSpecialCharacters() throws Exception {
        User paramOne = new User("peppy*pa%", "#pat-el-", "mumbai", "INDIA", "indian");
        assertEquals(paramOne.firstName, "peppypa");
        assertEquals(paramOne.lastName, "pat-el");
    }

    @Test
    public void shouldAppendLettersIfRequired() throws Exception {
        User paramOne = new User("pe%", "pa", "mumbai", "INDIA", "indian");
        assertEquals(paramOne.firstName, "pepe");
        assertEquals(paramOne.lastName, "papa");
    }

    @Test
    public void truncateLettersIfRequired() throws Exception {
        User paramOne = new User("suchalongnameuhavegiven", "pa", "mumbai", "INDIA", "indian");
        assertEquals(paramOne.firstName, "suchalongnameuh");
        assertEquals(paramOne.lastName, "papa");
    }

    @Test
    public void stringsWithoutSpecialCharsAreLeftUnchanged() {
        assertEquals("Naresh", stripSpecialCharacters("Naresh", allowedChars));
    }

    @Test
    public void specialCharsAtTheBeginningOfStringAreStrippedOut() {
        assertEquals("Naresh", stripSpecialCharacters("-@#Naresh", allowedChars));
    }

    @Test
    public void specialCharsAtTheEndOfStringAreStrippedOut() {
        assertEquals("Naresh", stripSpecialCharacters("Naresh#--", allowedChars));
    }

    @Test
    public void specialCharsAnyWhereInTheStringAreStrippedOut() {
        assertEquals("123@-Naresh-Jain", stripSpecialCharacters("-123@-Naresh #-!$ Jain-", allowedChars));
    }

}
